@@ -202,6 +202,35 @@ class ParseConnectTest(fixtures.TestBase):
 
         eq_(dialect.is_disconnect("not an error", None, None), False)
 
+    def test_pyodbc_disconnect(self):
+        dialect = pyodbc.dialect()
+
+        class MockDBAPIError(Exception):
+            pass
+
+        class MockProgrammingError(MockDBAPIError):
+            pass
+
+        dialect.dbapi = Mock(
+            Error=MockDBAPIError, ProgrammingError=MockProgrammingError)
+
+        for error in [
+            MockDBAPIError("[%s] some pyodbc message" % code)
+            for code in [
+                '08S01', '01002', '08003', '08007',
+                '08S02', '08001', 'HYT00', 'HY010']
+        ] + [
+            MockProgrammingError(message)
+            for message in [
+                "(some pyodbc stuff) The cursor's connection has been closed.",
+                "(some pyodbc stuff) Attempt to use a closed connection."
+            ]
+        ]:
+            eq_(dialect.is_disconnect(error, None, None), True)
+
+        eq_(dialect.is_disconnect(
+            MockProgrammingError("not an error"), None, None), False)
+
     @testing.requires.mssql_freetds
     def test_bad_freetds_warning(self):
         engine = engines.testing_engine()
@@ -242,3 +271,34 @@ class VersionDetectionTest(fixtures.TestBase):
                 dialect._get_server_version_info(conn),
                 (11, 0, 9216, 62)
             )
+
+    def test_pyodbc_version_productversion(self):
+        dialect = pyodbc.MSDialect_pyodbc()
+
+        conn = Mock(scalar=Mock(return_value="11.0.9216.62"))
+        eq_(
+            dialect._get_server_version_info(conn),
+            (11, 0, 9216, 62)
+        )
+
+    def test_pyodbc_version_fallback(self):
+        dialect = pyodbc.MSDialect_pyodbc()
+        dialect.dbapi = Mock()
+
+        for vers, expected in [
+            ("11.0.9216.62", (11, 0, 9216, 62)),
+            ("notsqlserver.11.foo.0.9216.BAR.62", (11, 0, 9216, 62)),
+            ("Not SQL Server Version 10.5", (5, ))
+        ]:
+            conn = Mock(
+                scalar=Mock(
+                    side_effect=exc.DBAPIError("stmt", "params", None)),
+                connection=Mock(
+                    getinfo=Mock(return_value=vers)
+                )
+            )
+
+            eq_(
+                dialect._get_server_version_info(conn),
+                expected
+            )
\ No newline at end of file
