@@ -8,14 +8,13 @@ from sqlalchemy import Table, MetaData, Column, select, String, \
     NUMERIC, DECIMAL, Numeric, Float, FLOAT, TIMESTAMP, DATE, \
     DATETIME, TIME, \
     DateTime, Time, Date, Interval, NCHAR, CHAR, CLOB, TEXT, Boolean, \
-    BOOLEAN, LargeBinary, BLOB, SmallInteger, INT, func, cast
+    BOOLEAN, LargeBinary, BLOB, SmallInteger, INT, func, cast, literal
 
+from sqlalchemy.dialects.mysql import insert
 from sqlalchemy.dialects.mysql import base as mysql
 from sqlalchemy.testing import fixtures, AssertsCompiledSQL
-from sqlalchemy.testing import mock
-from sqlalchemy import testing
 from sqlalchemy.sql import table, column
-import re
+from sqlalchemy.sql.expression import literal_column
 
 
 class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
@@ -203,6 +202,22 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             matchtable.c.title.match('somstr'),
             "MATCH (matchtable.title) AGAINST (%s IN BOOLEAN MODE)")
 
+    def test_match_compile_kw(self):
+        expr = literal('x').match(literal('y'))
+        self.assert_compile(
+            expr,
+            "MATCH ('x') AGAINST ('y' IN BOOLEAN MODE)",
+            literal_binds=True
+        )
+
+    def test_concat_compile_kw(self):
+        expr = literal('x', type_=String) + literal('y', type_=String)
+        self.assert_compile(
+            expr,
+            "concat('x', 'y')",
+            literal_binds=True
+        )
+
     def test_for_update(self):
         table1 = table('mytable',
                        column('myid'), column('name'), column('description'))
@@ -217,6 +232,23 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "SELECT mytable.myid, mytable.name, mytable.description "
             "FROM mytable WHERE mytable.myid = %s LOCK IN SHARE MODE")
 
+    def test_delete_extra_froms(self):
+        t1 = table('t1', column('c1'))
+        t2 = table('t2', column('c1'))
+        q = sql.delete(t1).where(t1.c.c1 == t2.c.c1)
+        self.assert_compile(
+            q, "DELETE FROM t1 USING t1, t2 WHERE t1.c1 = t2.c1"
+        )
+
+    def test_delete_extra_froms_alias(self):
+        a1 = table('t1', column('c1')).alias('a1')
+        t2 = table('t2', column('c1'))
+        q = sql.delete(a1).where(a1.c.c1 == t2.c.c1)
+        self.assert_compile(
+            q, "DELETE FROM a1 USING t1 AS a1, t2 WHERE a1.c1 = t2.c1"
+        )
+        self.assert_compile(sql.delete(a1), "DELETE FROM t1 AS a1")
+
 
 class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
 
@@ -691,3 +723,46 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             t1.outerjoin(t2, t1.c.x == t2.c.y, full=True),
             "t1 FULL OUTER JOIN t2 ON t1.x = t2.y"
         )
+
+
+class InsertOnDuplicateTest(fixtures.TestBase, AssertsCompiledSQL):
+    __dialect__ = mysql.dialect()
+
+    def setup(self):
+        self.table = Table(
+            'foos', MetaData(),
+            Column('id', Integer, primary_key=True),
+            Column('bar', String(10)),
+            Column('baz', String(10)),
+        )
+
+    def test_from_values(self):
+        stmt = insert(self.table).values(
+            [{'id': 1, 'bar': 'ab'}, {'id': 2, 'bar': 'b'}])
+        stmt = stmt.on_duplicate_key_update(
+            bar=stmt.inserted.bar, baz=stmt.inserted.baz)
+        expected_sql = (
+            'INSERT INTO foos (id, bar) VALUES (%s, %s), (%s, %s) '
+            'ON DUPLICATE KEY UPDATE bar = VALUES(bar), baz = VALUES(baz)'
+        )
+        self.assert_compile(stmt, expected_sql)
+
+    def test_from_literal(self):
+        stmt = insert(self.table).values(
+            [{'id': 1, 'bar': 'ab'}, {'id': 2, 'bar': 'b'}])
+        stmt = stmt.on_duplicate_key_update(bar=literal_column('bb'))
+        expected_sql = (
+            'INSERT INTO foos (id, bar) VALUES (%s, %s), (%s, %s) '
+            'ON DUPLICATE KEY UPDATE bar = bb'
+        )
+        self.assert_compile(stmt, expected_sql)
+
+    def test_python_values(self):
+        stmt = insert(self.table).values(
+            [{'id': 1, 'bar': 'ab'}, {'id': 2, 'bar': 'b'}])
+        stmt = stmt.on_duplicate_key_update(bar="foobar")
+        expected_sql = (
+            'INSERT INTO foos (id, bar) VALUES (%s, %s), (%s, %s) '
+            'ON DUPLICATE KEY UPDATE bar = %s'
+        )
+        self.assert_compile(stmt, expected_sql)
