@@ -32,3 +32,20 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "Sybase ASE does not support OFFSET",
             stmt.compile, dialect=self.__dialect__
         )
+
+    def test_delete_extra_froms(self):
+        t1 = sql.table('t1', sql.column('c1'))
+        t2 = sql.table('t2', sql.column('c1'))
+        q = sql.delete(t1).where(t1.c.c1 == t2.c.c1)
+        self.assert_compile(
+            q, "DELETE FROM t1 FROM t1, t2 WHERE t1.c1 = t2.c1"
+        )
+
+    def test_delete_extra_froms_alias(self):
+        a1 = sql.table('t1', sql.column('c1')).alias('a1')
+        t2 = sql.table('t2', sql.column('c1'))
+        q = sql.delete(a1).where(a1.c.c1 == t2.c.c1)
+        self.assert_compile(
+            q, "DELETE FROM a1 FROM t1 AS a1, t2 WHERE a1.c1 = t2.c1"
+        )
+        self.assert_compile(sql.delete(a1), "DELETE FROM t1 AS a1")
