@@ -1,12 +1,15 @@
 # coding: utf-8
 
+import weakref
 from sqlalchemy.testing import eq_, assert_raises, assert_raises_message, \
     config, is_, is_not_, le_, expect_warnings
 import re
 from sqlalchemy.testing.util import picklers
+from sqlalchemy.testing.util import gc_collect
 from sqlalchemy.interfaces import ConnectionProxy
 from sqlalchemy import MetaData, Integer, String, INT, VARCHAR, func, \
-    bindparam, select, event, TypeDecorator, create_engine, Sequence
+    bindparam, select, event, TypeDecorator, create_engine, Sequence, \
+    LargeBinary
 from sqlalchemy.sql import column, literal
 from sqlalchemy.testing.schema import Table, Column
 import sqlalchemy as tsa
@@ -19,6 +22,7 @@ from sqlalchemy.engine import result as _result, default
 from sqlalchemy.engine.base import Engine
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing.mock import Mock, call, patch
+from sqlalchemy.testing import mock
 from contextlib import contextmanager
 from sqlalchemy.util import nested
 from sqlalchemy.testing.assertsql import CompiledSQL
@@ -41,7 +45,7 @@ class ExecuteTest(fixtures.TestBase):
         users = Table(
             'users', metadata,
             Column('user_id', INT, primary_key=True, autoincrement=False),
-            Column('user_name', VARCHAR(20)),
+            Column('user_name', VARCHAR(20))
         )
         users_autoinc = Table(
             'users_autoinc', metadata,
@@ -178,7 +182,8 @@ class ExecuteTest(fixtures.TestBase):
     @testing.fails_on_everything_except(
         'postgresql+psycopg2', 'postgresql+psycopg2cffi',
         'postgresql+pypostgresql', 'postgresql+pygresql',
-        'mysql+mysqlconnector', 'mysql+pymysql', 'mysql+cymysql')
+        'mysql+mysqlconnector', 'mysql+pymysql', 'mysql+cymysql',
+        'mssql+pymssql')
     def test_raw_python(self):
         def go(conn):
             conn.execute(
@@ -433,6 +438,35 @@ class ExecuteTest(fixtures.TestBase):
                            values(user_name=bindparam('name', None)), [])
         eq_(testing.db.execute(users_autoinc.select()).fetchall(), [(1, None)])
 
+    @testing.only_on("sqlite")
+    def test_execute_compiled_favors_compiled_paramstyle(self):
+        with patch.object(testing.db.dialect, "do_execute") as do_exec:
+            stmt = users.update().values(user_id=1, user_name='foo')
+
+            d1 = default.DefaultDialect(paramstyle="format")
+            d2 = default.DefaultDialect(paramstyle="pyformat")
+
+            testing.db.execute(stmt.compile(dialect=d1))
+            testing.db.execute(stmt.compile(dialect=d2))
+
+            eq_(
+                do_exec.mock_calls, [
+                    call(
+                        mock.ANY,
+                        "UPDATE users SET user_id=%s, user_name=%s",
+                        (1, 'foo'),
+                        mock.ANY
+                    ),
+                    call(
+                        mock.ANY,
+                        "UPDATE users SET user_id=%(user_id)s, "
+                        "user_name=%(user_name)s",
+                       {'user_name': 'foo', 'user_id': 1},
+                        mock.ANY
+                    )
+                ]
+            )
+
     @testing.requires.ad_hoc_engines
     def test_engine_level_options(self):
         eng = engines.testing_engine(options={'execution_options':
@@ -479,57 +513,6 @@ class ExecuteTest(fixtures.TestBase):
         is_(eng1a.pool, eng2.pool)
         is_(eng.pool, eng2.pool)
 
-    @testing.requires.ad_hoc_engines
-    def test_generative_engine_event_dispatch(self):
-        canary = []
-
-        def l1(*arg, **kw):
-            canary.append("l1")
-
-        def l2(*arg, **kw):
-            canary.append("l2")
-
-        def l3(*arg, **kw):
-            canary.append("l3")
-
-        eng = engines.testing_engine(options={'execution_options':
-                                              {'base': 'x1'}})
-        event.listen(eng, "before_execute", l1)
-
-        eng1 = eng.execution_options(foo="b1")
-        event.listen(eng, "before_execute", l2)
-        event.listen(eng1, "before_execute", l3)
-
-        eng.execute(select([1])).close()
-        eng1.execute(select([1])).close()
-
-        eq_(canary, ["l1", "l2", "l3", "l1", "l2"])
-
-    @testing.requires.ad_hoc_engines
-    def test_dispose_event(self):
-        canary = Mock()
-        eng = create_engine(testing.db.url)
-        event.listen(eng, "engine_disposed", canary)
-
-        conn = eng.connect()
-        conn.close()
-        eng.dispose()
-
-        conn = eng.connect()
-        conn.close()
-
-        eq_(
-            canary.mock_calls,
-            [call(eng)]
-        )
-
-        eng.dispose()
-
-        eq_(
-            canary.mock_calls,
-            [call(eng), call(eng)]
-        )
-
     @testing.requires.ad_hoc_engines
     def test_autocommit_option_no_issue_first_connect(self):
         eng = create_engine(testing.db.url)
@@ -790,6 +773,49 @@ class CompiledCacheTest(fixtures.TestBase):
         assert len(cache) == 1
         eq_(conn.execute("select count(*) from users").scalar(), 3)
 
+    @testing.only_on(["sqlite", "mysql", "postgresql"],
+                     "uses blob value that is problematic for some DBAPIs")
+    @testing.provide_metadata
+    def test_cache_noleak_on_statement_values(self):
+        # This is a non regression test for an object reference leak caused
+        # by the compiled_cache.
+
+        metadata = self.metadata
+        photo = Table(
+            'photo', metadata,
+            Column('id', Integer, primary_key=True,
+                   test_needs_autoincrement=True),
+            Column('photo_blob', LargeBinary()),
+        )
+        metadata.create_all()
+
+        conn = testing.db.connect()
+        cache = {}
+        cached_conn = conn.execution_options(compiled_cache=cache)
+
+        class PhotoBlob(bytearray):
+            pass
+
+        blob = PhotoBlob(100)
+        ref_blob = weakref.ref(blob)
+
+        ins = photo.insert()
+        with patch.object(
+            ins, "compile",
+                Mock(side_effect=ins.compile)) as compile_mock:
+            cached_conn.execute(ins, {'photo_blob': blob})
+        eq_(compile_mock.call_count, 1)
+        eq_(len(cache), 1)
+        eq_(conn.execute("select count(*) from photo").scalar(), 1)
+
+        del blob
+
+        gc_collect()
+
+        # The compiled statement cache should not hold any reference to the
+        # the statement values (only the keys).
+        eq_(ref_blob(), None)
+
     def test_keys_independent_of_ordering(self):
         conn = testing.db.connect()
         conn.execute(
@@ -1320,34 +1346,19 @@ class EngineEventsTest(fixtures.TestBase):
                         ('select * from t1', {}, None),
                         ('DROP TABLE t1', {}, None)]
 
-            # or engine.dialect.preexecute_pk_sequences:
-            if not testing.against('oracle+zxjdbc'):
-                cursor = [
-                    ('CREATE TABLE t1', {}, ()),
-                    ('INSERT INTO t1 (c1, c2)', {
-                        'c2': 'some data', 'c1': 5},
-                        (5, 'some data')),
-                    ('SELECT lower', {'lower_2': 'Foo'},
-                        ('Foo', )),
-                    ('INSERT INTO t1 (c1, c2)',
-                     {'c2': 'foo', 'c1': 6},
-                     (6, 'foo')),
-                    ('select * from t1', {}, ()),
-                    ('DROP TABLE t1', {}, ()),
-                ]
-            else:
-                insert2_params = 6, 'Foo'
-                if testing.against('oracle+zxjdbc'):
-                    insert2_params += (ReturningParam(12), )
-                cursor = [('CREATE TABLE t1', {}, ()),
-                          ('INSERT INTO t1 (c1, c2)',
-                           {'c2': 'some data', 'c1': 5}, (5, 'some data')),
-                          ('INSERT INTO t1 (c1, c2)',
-                           {'c1': 6, 'lower_2': 'Foo'}, insert2_params),
-                          ('select * from t1', {}, ()),
-                          ('DROP TABLE t1', {}, ())]
-                # bind param name 'lower_2' might
-                # be incorrect
+            cursor = [
+                ('CREATE TABLE t1', {}, ()),
+                ('INSERT INTO t1 (c1, c2)', {
+                    'c2': 'some data', 'c1': 5},
+                    (5, 'some data')),
+                ('SELECT lower', {'lower_1': 'Foo'},
+                    ('Foo', )),
+                ('INSERT INTO t1 (c1, c2)',
+                 {'c2': 'foo', 'c1': 6},
+                 (6, 'foo')),
+                ('select * from t1', {}, ()),
+                ('DROP TABLE t1', {}, ()),
+            ]
             self._assert_stmts(compiled, stmts)
             self._assert_stmts(cursor, cursor_stmts)
 
@@ -1371,6 +1382,117 @@ class EngineEventsTest(fixtures.TestBase):
         eq_(c3._execution_options, {'foo': 'bar', 'bar': 'bat'})
         eq_(canary, ['execute', 'cursor_execute'])
 
+    @testing.requires.ad_hoc_engines
+    def test_generative_engine_event_dispatch(self):
+        canary = []
+
+        def l1(*arg, **kw):
+            canary.append("l1")
+
+        def l2(*arg, **kw):
+            canary.append("l2")
+
+        def l3(*arg, **kw):
+            canary.append("l3")
+
+        eng = engines.testing_engine(options={'execution_options':
+                                              {'base': 'x1'}})
+        event.listen(eng, "before_execute", l1)
+
+        eng1 = eng.execution_options(foo="b1")
+        event.listen(eng, "before_execute", l2)
+        event.listen(eng1, "before_execute", l3)
+
+        eng.execute(select([1])).close()
+
+        eq_(canary, ["l1", "l2"])
+
+        eng1.execute(select([1])).close()
+
+        eq_(canary, ["l1", "l2", "l3", "l1", "l2"])
+
+    @testing.requires.ad_hoc_engines
+    def test_clslevel_engine_event_options(self):
+        canary = []
+
+        def l1(*arg, **kw):
+            canary.append("l1")
+
+        def l2(*arg, **kw):
+            canary.append("l2")
+
+        def l3(*arg, **kw):
+            canary.append("l3")
+
+        def l4(*arg, **kw):
+            canary.append("l4")
+
+        event.listen(Engine, "before_execute", l1)
+
+        eng = engines.testing_engine(options={'execution_options':
+                                              {'base': 'x1'}})
+        event.listen(eng, "before_execute", l2)
+
+        eng1 = eng.execution_options(foo="b1")
+        event.listen(eng, "before_execute", l3)
+        event.listen(eng1, "before_execute", l4)
+
+        eng.execute(select([1])).close()
+
+        eq_(canary, ["l1", "l2", "l3"])
+
+        eng1.execute(select([1])).close()
+
+        eq_(canary, ["l1", "l2", "l3", "l4", "l1", "l2", "l3"])
+
+        canary[:] = []
+
+        event.remove(Engine, "before_execute", l1)
+        event.remove(eng1, "before_execute", l4)
+        event.remove(eng, "before_execute", l3)
+
+        eng1.execute(select([1])).close()
+        eq_(canary, ["l2"])
+
+    @testing.requires.ad_hoc_engines
+    def test_cant_listen_to_option_engine(self):
+        from sqlalchemy.engine import base
+
+        def evt(*arg, **kw):
+            pass
+
+        assert_raises_message(
+            tsa.exc.InvalidRequestError,
+            r"Can't assign an event directly to the "
+            "<class 'sqlalchemy.engine.base.OptionEngine'> class",
+            event.listen, base.OptionEngine, "before_cursor_execute", evt
+        )
+
+    @testing.requires.ad_hoc_engines
+    def test_dispose_event(self):
+        canary = Mock()
+        eng = create_engine(testing.db.url)
+        event.listen(eng, "engine_disposed", canary)
+
+        conn = eng.connect()
+        conn.close()
+        eng.dispose()
+
+        conn = eng.connect()
+        conn.close()
+
+        eq_(
+            canary.mock_calls,
+            [call(eng)]
+        )
+
+        eng.dispose()
+
+        eq_(
+            canary.mock_calls,
+            [call(eng), call(eng)]
+        )
+
     def test_retval_flag(self):
         canary = []
 
@@ -1813,7 +1935,8 @@ class HandleErrorTest(fixtures.TestBase):
 
         with expect_warnings(
             r"An exception has occurred during handling of a previous "
-            r"exception.  The previous exception is.*i_dont_exist",
+            r"exception.  The previous exception "
+            r"is.*(?:i_dont_exist|does not exist)",
             py2konly=True
         ):
             with patch.object(conn.dialect, "do_rollback", boom) as patched:
@@ -2333,31 +2456,19 @@ class ProxyConnectionTest(fixtures.TestBase):
                         ('INSERT INTO t1 (c1, c2)', {'c1': 6}, None),
                         ('select * from t1', {}, None),
                         ('DROP TABLE t1', {}, None)]
-            # or engine.dialect.pr eexecute_pk_sequence s:
-            # original comment above moved here for pep8 fix
-            if not testing.against('oracle+zxjdbc'):
-                cursor = [
-                    ('CREATE TABLE t1', {}, ()),
-                    ('INSERT INTO t1 (c1, c2)', {
-                     'c2': 'some data', 'c1': 5}, (5, 'some data')),
-                    ('SELECT lower', {'lower_2': 'Foo'},
-                        ('Foo', )),
-                    ('INSERT INTO t1 (c1, c2)', {'c2': 'foo', 'c1': 6},
-                     (6, 'foo')),
-                    ('select * from t1', {}, ()),
-                    ('DROP TABLE t1', {}, ()),
-                ]
-            else:
-                insert2_params = 6, 'Foo'
-                if testing.against('oracle+zxjdbc'):
-                    insert2_params += (ReturningParam(12), )
-                cursor = [('CREATE TABLE t1', {}, ()),
-                          ('INSERT INTO t1 (c1, c2)', {
-                           'c2': 'some data', 'c1': 5}, (5, 'some data')),
-                          ('INSERT INTO t1 (c1, c2)',
-                           {'c1': 6, 'lower_2': 'Foo'}, insert2_params),
-                          ('select * from t1', {}, ()),
-                          ('DROP TABLE t1', {}, ())]
+
+            cursor = [
+                ('CREATE TABLE t1', {}, ()),
+                ('INSERT INTO t1 (c1, c2)', {
+                 'c2': 'some data', 'c1': 5}, (5, 'some data')),
+                ('SELECT lower', {'lower_1': 'Foo'},
+                    ('Foo', )),
+                ('INSERT INTO t1 (c1, c2)', {'c2': 'foo', 'c1': 6},
+                 (6, 'foo')),
+                ('select * from t1', {}, ()),
+                ('DROP TABLE t1', {}, ()),
+            ]
+
             assert_stmts(compiled, stmts)
             assert_stmts(cursor, cursor_stmts)
 
