@@ -1,5 +1,5 @@
 from sqlalchemy.testing import eq_, assert_raises, \
-    assert_raises_message, is_
+    assert_raises_message, is_, expect_warnings
 from sqlalchemy.ext import declarative as decl
 import sqlalchemy as sa
 from sqlalchemy import testing
@@ -1188,6 +1188,69 @@ class DeclarativeMixinTest(DeclarativeTestBase):
         TypeA(filters=['foo'])
         TypeB(filters=['foo'])
 
+    def test_arbitrary_attrs_three(self):
+        class Mapped(Base):
+            __tablename__ = 't'
+            id = Column(Integer, primary_key=True)
+
+            @declared_attr
+            def some_attr(cls):
+                return cls.__name__ + "SOME ATTR"
+
+        eq_(Mapped.some_attr, "MappedSOME ATTR")
+        eq_(Mapped.__dict__['some_attr'], "MappedSOME ATTR")
+
+    def test_arbitrary_attrs_doesnt_apply_to_abstract_declared_attr(self):
+        names = ["name1", "name2", "name3"]
+
+        class SomeAbstract(Base):
+            __abstract__ = True
+
+            @declared_attr
+            def some_attr(cls):
+                return names.pop(0)
+
+        class M1(SomeAbstract):
+            __tablename__ = 't1'
+            id = Column(Integer, primary_key=True)
+
+        class M2(SomeAbstract):
+            __tablename__ = 't2'
+            id = Column(Integer, primary_key=True)
+
+        eq_(M1.__dict__['some_attr'], 'name1')
+        eq_(M2.__dict__['some_attr'], 'name2')
+
+    def test_arbitrary_attrs_doesnt_apply_to_prepare_nocascade(self):
+        names = ["name1", "name2", "name3"]
+
+        class SomeAbstract(Base):
+            __tablename__ = 't0'
+            __no_table__ = True
+
+            # used by AbstractConcreteBase
+            _sa_decl_prepare_nocascade = True
+
+            id = Column(Integer, primary_key=True)
+
+            @declared_attr
+            def some_attr(cls):
+                return names.pop(0)
+
+        class M1(SomeAbstract):
+            __tablename__ = 't1'
+            id = Column(Integer, primary_key=True)
+
+        class M2(SomeAbstract):
+            __tablename__ = 't2'
+            id = Column(Integer, primary_key=True)
+
+        eq_(M1.some_attr, "name2")
+        eq_(M2.some_attr, "name3")
+        eq_(M1.__dict__['some_attr'], 'name2')
+        eq_(M2.__dict__['some_attr'], 'name3')
+        assert isinstance(SomeAbstract.__dict__['some_attr'], declared_attr)
+
 
 class DeclarativeMixinPropertyTest(DeclarativeTestBase):
 
@@ -1477,6 +1540,19 @@ class DeclaredAttrTest(DeclarativeTestBase, testing.AssertsCompiledSQL):
             getattr, Mixin, "my_prop"
         )
 
+    def test_can_we_access_the_mixin_straight_special_names(self):
+        class Mixin(object):
+            @declared_attr
+            def __table_args__(cls):
+                return (1, 2, 3)
+
+            @declared_attr
+            def __arbitrary__(cls):
+                return (4, 5, 6)
+
+        eq_(Mixin.__table_args__, (1, 2, 3))
+        eq_(Mixin.__arbitrary__, (4, 5, 6))
+
     def test_non_decl_access(self):
         counter = mock.Mock()
 
@@ -1530,7 +1606,7 @@ class DeclaredAttrTest(DeclarativeTestBase, testing.AssertsCompiledSQL):
 
         eq_(counter.mock_calls, [mock.call(A)])
 
-    def test_property_cascade(self):
+    def test_property_cascade_mixin(self):
         counter = mock.Mock()
 
         class Mixin(object):
@@ -1550,6 +1626,67 @@ class DeclaredAttrTest(DeclarativeTestBase, testing.AssertsCompiledSQL):
 
         eq_(counter.mock_calls, [mock.call(A), mock.call(B)])
 
+    def test_property_cascade_mixin_override(self):
+        counter = mock.Mock()
+
+        class Mixin(object):
+            @declared_attr.cascading
+            def my_prop(cls):
+                counter(cls)
+                return column_property(cls.x + 2)
+
+        class A(Base, Mixin):
+            __tablename__ = 'a'
+
+            id = Column(Integer, primary_key=True)
+            x = Column(Integer)
+
+        with expect_warnings(
+                "Attribute 'my_prop' on class .*B.* "
+                "cannot be processed due to @declared_attr.cascading; "
+                "skipping"):
+            class B(A):
+                my_prop = Column('foobar', Integer)
+
+        eq_(counter.mock_calls, [mock.call(A), mock.call(B)])
+
+    def test_property_cascade_abstract(self):
+        counter = mock.Mock()
+
+        class Abs(Base):
+            __abstract__ = True
+
+            @declared_attr.cascading
+            def my_prop(cls):
+                counter(cls)
+                return column_property(cls.x + 2)
+
+        class A(Abs):
+            __tablename__ = 'a'
+
+            id = Column(Integer, primary_key=True)
+            x = Column(Integer)
+
+        class B(A):
+            pass
+
+        eq_(counter.mock_calls, [mock.call(A), mock.call(B)])
+
+    def test_warn_cascading_used_w_tablename(self):
+        class Mixin(object):
+            @declared_attr.cascading
+            def __tablename__(cls):
+                return "foo"
+
+        with expect_warnings(
+            "@declared_attr.cascading is not supported on the "
+            "__tablename__ attribute on class .*A."
+        ):
+            class A(Mixin, Base):
+                id = Column(Integer, primary_key=True)
+
+        eq_(A.__table__.name, "foo")
+
     def test_col_prop_attrs_associated_w_class_for_mapper_args(self):
         from sqlalchemy import Column
         import collections
