@@ -2,6 +2,7 @@ from sqlalchemy import Integer, ForeignKey, String, func
 from sqlalchemy.types import PickleType, TypeDecorator, VARCHAR
 from sqlalchemy.orm import mapper, Session, composite, column_property
 from sqlalchemy.orm.mapper import Mapper
+from sqlalchemy.orm import attributes
 from sqlalchemy.orm.instrumentation import ClassManager
 from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.testing import eq_, assert_raises_message, assert_raises
@@ -9,6 +10,8 @@ from sqlalchemy.testing.util import picklers
 from sqlalchemy.testing import fixtures
 from sqlalchemy.ext.mutable import MutableComposite
 from sqlalchemy.ext.mutable import MutableDict, MutableList, MutableSet
+from sqlalchemy.testing import mock
+from sqlalchemy import event
 
 
 class Foo(fixtures.BasicEntity):
@@ -112,6 +115,19 @@ class _MutableDictTestBase(_MutableDictTestFixture):
 
         eq_(f1.data, {'a': 'c'})
 
+    def test_modified_event(self):
+        canary = mock.Mock()
+        event.listen(Foo.data, "modified", canary)
+
+        f1 = Foo(data={"a": "b"})
+        f1.data["a"] = "c"
+
+        eq_(
+            canary.mock_calls,
+            [mock.call(
+                f1, attributes.Event(Foo.data.impl, attributes.OP_MODIFIED))]
+        )
+
     def test_clear(self):
         sess = Session()
 
@@ -385,6 +401,18 @@ class _MutableListTestBase(_MutableListTestFixture):
 
         eq_(f1.data, [1, 2, 5])
 
+    def test_operator_extend(self):
+        sess = Session()
+
+        f1 = Foo(data=[1, 2])
+        sess.add(f1)
+        sess.commit()
+
+        f1.data += [5]
+        sess.commit()
+
+        eq_(f1.data, [1, 2, 5])
+
     def test_insert(self):
         sess = Session()
 
@@ -545,6 +573,18 @@ class _MutableSetTestBase(_MutableSetTestFixture):
 
         eq_(f1.data, set([1, 2, 5]))
 
+    def test_binary_update(self):
+        sess = Session()
+
+        f1 = Foo(data=set([1, 2]))
+        sess.add(f1)
+        sess.commit()
+
+        f1.data |= set([2, 5])
+        sess.commit()
+
+        eq_(f1.data, set([1, 2, 5]))
+
     def test_intersection_update(self):
         sess = Session()
 
@@ -557,6 +597,18 @@ class _MutableSetTestBase(_MutableSetTestFixture):
 
         eq_(f1.data, set([2]))
 
+    def test_binary_intersection_update(self):
+        sess = Session()
+
+        f1 = Foo(data=set([1, 2]))
+        sess.add(f1)
+        sess.commit()
+
+        f1.data &= set([2, 5])
+        sess.commit()
+
+        eq_(f1.data, set([2]))
+
     def test_difference_update(self):
         sess = Session()
 
@@ -569,6 +621,18 @@ class _MutableSetTestBase(_MutableSetTestFixture):
 
         eq_(f1.data, set([1]))
 
+    def test_operator_difference_update(self):
+        sess = Session()
+
+        f1 = Foo(data=set([1, 2]))
+        sess.add(f1)
+        sess.commit()
+
+        f1.data -= set([2, 5])
+        sess.commit()
+
+        eq_(f1.data, set([1]))
+
     def test_symmetric_difference_update(self):
         sess = Session()
 
@@ -581,6 +645,18 @@ class _MutableSetTestBase(_MutableSetTestFixture):
 
         eq_(f1.data, set([1, 5]))
 
+    def test_binary_symmetric_difference_update(self):
+        sess = Session()
+
+        f1 = Foo(data=set([1, 2]))
+        sess.add(f1)
+        sess.commit()
+
+        f1.data ^= set([2, 5])
+        sess.commit()
+
+        eq_(f1.data, set([1, 5]))
+
     def test_remove(self):
         sess = Session()
 
@@ -725,6 +801,17 @@ class MutableWithScalarJSONTest(_MutableDictTestBase, fixtures.MappedTest):
         self._test_non_mutable()
 
 
+class MutableIncludeNonPrimaryTest(MutableWithScalarJSONTest):
+    @classmethod
+    def setup_mappers(cls):
+        foo = cls.tables.foo
+
+        mapper(Foo, foo)
+        mapper(Foo, foo, non_primary=True, properties={
+            "foo_bar": foo.c.data
+        })
+
+
 class MutableColumnCopyJSONTest(_MutableDictTestBase, fixtures.MappedTest):
 
     @classmethod
@@ -770,6 +857,26 @@ class MutableColumnCopyJSONTest(_MutableDictTestBase, fixtures.MappedTest):
         self._test_non_mutable()
 
 
+class MutableColumnCopyArrayTest(_MutableListTestBase, fixtures.MappedTest):
+    __requires__ = 'array_type',
+
+    @classmethod
+    def define_tables(cls, metadata):
+        from sqlalchemy.ext.declarative import declarative_base
+        from sqlalchemy.sql.sqltypes import ARRAY
+
+        MutableList = cls._type_fixture()
+
+        Base = declarative_base(metadata=metadata)
+
+        class Mixin(object):
+            data = Column(MutableList.as_mutable(ARRAY(Integer)))
+
+        class Foo(Mixin, Base):
+            __tablename__ = 'foo'
+            id = Column(Integer, primary_key=True)
+
+
 class MutableListWithScalarPickleTest(_MutableListTestBase,
                                       fixtures.MappedTest):
 
@@ -871,6 +978,17 @@ class MutableAssociationScalarPickleTest(_MutableDictTestBase,
               )
 
 
+class MutableAssocIncludeNonPrimaryTest(MutableAssociationScalarPickleTest):
+    @classmethod
+    def setup_mappers(cls):
+        foo = cls.tables.foo
+
+        mapper(Foo, foo)
+        mapper(Foo, foo, non_primary=True, properties={
+            "foo_bar": foo.c.data
+        })
+
+
 class MutableAssociationScalarJSONTest(_MutableDictTestBase,
                                        fixtures.MappedTest):
 
