@@ -247,6 +247,25 @@ class SingleInheritanceTest(testing.AssertsCompiledSQL, fixtures.MappedTest):
                             'anon_1',
                             use_default_dialect=True)
 
+    def test_select_from_aliased_w_subclass(self):
+        Engineer = self.classes.Engineer
+
+        sess = create_session()
+
+        a1 = aliased(Engineer)
+        self.assert_compile(
+            sess.query(a1.employee_id).select_from(a1),
+            "SELECT employees_1.employee_id AS employees_1_employee_id "
+            "FROM employees AS employees_1 WHERE employees_1.type "
+            "IN (:type_1, :type_2)",
+        )
+
+        self.assert_compile(
+            sess.query(literal('1')).select_from(a1),
+            "SELECT :param_1 AS param_1 FROM employees AS employees_1 "
+            "WHERE employees_1.type IN (:type_1, :type_2)"
+        )
+
     def test_union_modifiers(self):
         Engineer, Manager = self.classes("Engineer", "Manager")
 
@@ -310,11 +329,26 @@ class SingleInheritanceTest(testing.AssertsCompiledSQL, fixtures.MappedTest):
             use_default_dialect=True
         )
 
-    def test_select_from(self):
-        Manager = self.classes.Manager
-        JuniorEngineer = self.classes.JuniorEngineer
-        employees = self.tables.employees
-        Engineer = self.classes.Engineer
+    def test_select_from_count(self):
+        Manager, Engineer = (self.classes.Manager, self.classes.Engineer)
+
+        sess = create_session()
+        m1 = Manager(name='Tom', manager_data='data1')
+        e1 = Engineer(name='Kurt', engineer_info='knows how to hack')
+        sess.add_all([m1, e1])
+        sess.flush()
+
+        eq_(
+            sess.query(func.count(1)).select_from(Manager).all(),
+            [(1, )]
+        )
+
+    def test_select_from_subquery(self):
+        Manager, JuniorEngineer, employees, Engineer = (
+            self.classes.Manager,
+            self.classes.JuniorEngineer,
+            self.tables.employees,
+            self.classes.Engineer)
 
         sess = create_session()
         m1 = Manager(name='Tom', manager_data='data1')
@@ -754,6 +788,38 @@ class RelationshipToSingleTest(
             "AND employees_1.type IN (:type_1)"
         )
 
+    def test_correlated_column_select(self):
+        Company, Employee, Engineer = (self.classes.Company,
+                                       self.classes.Employee,
+                                       self.classes.Engineer)
+        companies, employees = self.tables.companies, self.tables.employees
+
+        mapper(Company, companies)
+        mapper(
+            Employee, employees,
+            polymorphic_on=employees.c.type,
+            properties={
+                'company': relationship(Company)
+            }
+        )
+        mapper(Engineer, inherits=Employee, polymorphic_identity='engineer')
+
+        sess = create_session()
+        engineer_count = sess.query(func.count(Engineer.employee_id)) \
+            .select_from(Engineer) \
+            .filter(Engineer.company_id == Company.company_id) \
+            .correlate(Company) \
+            .as_scalar()
+
+        self.assert_compile(
+            sess.query(Company.company_id, engineer_count),
+            "SELECT companies.company_id AS companies_company_id, "
+            "(SELECT count(employees.employee_id) AS count_1 "
+            "FROM employees WHERE employees.company_id = "
+            "companies.company_id AND employees.type IN (:type_1)) AS anon_1 "
+            "FROM companies"
+        )
+
     def test_no_aliasing_from_overlap(self):
         # test [ticket:3233]
 
