@@ -8,6 +8,7 @@ from sqlalchemy.testing import fixtures, AssertsExecutionResults, \
 from sqlalchemy import testing
 from sqlalchemy.sql.visitors import ClauseVisitor, CloningVisitor, \
     cloned_traverse, ReplacingCloningVisitor
+from sqlalchemy.sql import visitors
 from sqlalchemy import exc
 from sqlalchemy.sql import util as sql_util
 from sqlalchemy.testing import (eq_,
@@ -496,6 +497,22 @@ class ClauseTest(fixtures.TestBase, AssertsCompiledSQL):
                             "SELECT sum(t.n) AS sum_1 FROM t"
                             )
 
+    def test_aliased_cte_w_union(self):
+        t = select([func.values(1).label("n")]).\
+            cte("t", recursive=True).alias('foo')
+        t = t.union_all(select([t.c.n + 1]).where(t.c.n < 100))
+        s = select([func.sum(t.c.n)])
+
+        from sqlalchemy.sql.visitors import cloned_traverse
+        cloned = cloned_traverse(s, {}, {})
+
+        self.assert_compile(
+            cloned,
+            "WITH RECURSIVE foo(n) AS (SELECT values(:values_1) AS n "
+            "UNION ALL SELECT foo.n + :n_1 AS anon_1 FROM foo "
+            "WHERE foo.n < :n_2) SELECT sum(foo.n) AS sum_1 FROM foo"
+        )
+
     def test_text(self):
         clause = text(
             "select * from table where foo=:bar",
@@ -594,6 +611,15 @@ class ClauseTest(fixtures.TestBase, AssertsCompiledSQL):
         expr2 = CloningVisitor().traverse(expr)
         assert str(expr) == str(expr2)
 
+        assert expr in visitors.iterate(expr, {})
+
+    def test_within_group(self):
+        expr = func.row_number().within_group(t1.c.col1)
+        expr2 = CloningVisitor().traverse(expr)
+        assert str(expr) == str(expr2)
+
+        assert expr in visitors.iterate(expr, {})
+
     def test_funcfilter(self):
         expr = func.count(1).filter(t1.c.col1 > 1)
         expr2 = CloningVisitor().traverse(expr)
